@@ -41,10 +41,12 @@
 #include "hfp.h"
 #include "slc.h"
 
+static const char *none_prefix[] = { NULL };
 static const char *brsf_prefix[] = { "+BRSF:", NULL };
 static const char *cind_prefix[] = { "+CIND:", NULL };
 static const char *cmer_prefix[] = { "+CMER:", NULL };
 static const char *chld_prefix[] = { "+CHLD:", NULL };
+static const char *bind_prefix[] = { "+BIND:", NULL };
 
 struct slc_establish_data {
 	gint ref_count;
@@ -76,6 +78,14 @@ void hfp_slc_info_init(struct hfp_slc_info *info, guint16 version)
 
 	info->hf_features |= HFP_HF_FEATURE_CODEC_NEGOTIATION;
 
+	if (version < HFP_VERSION_1_7)
+		goto done;
+
+	info->hf_features |= HFP_HF_FEATURE_HF_INDICATORS;
+	memset(info->hf_indicators, 0, sizeof(info->hf_indicators));
+	info->num_hf_indicators = 0;
+	info->hf_indicator_active_map = 0;
+
 done:
 	memset(info->cind_val, 0, sizeof(info->cind_val));
 	memset(info->cind_pos, 0, sizeof(info->cind_pos));
@@ -103,10 +113,109 @@ static void slc_established(struct slc_establish_data *sed)
 {
 	struct hfp_slc_info *info = sed->info;
 
-	g_at_chat_send(info->chat, "AT+CMEE=1", NULL, NULL, NULL, NULL);
+	g_at_chat_send(info->chat, "AT+CMEE=1", none_prefix,
+			NULL, NULL, NULL);
 	sed->connect_cb(sed->userdata);
 }
 
+static void bind_query_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct slc_establish_data *sed = user_data;
+	struct hfp_slc_info *info = sed->info;
+	GAtResultIter iter;
+	int hf_indicator;
+	int enabled;
+	unsigned int i;
+
+	if (!ok)
+		goto error;
+
+	g_at_result_iter_init(&iter, result);
+
+	while (g_at_result_iter_next(&iter, "+BIND:")) {
+		if (!g_at_result_iter_next_number(&iter, &hf_indicator))
+			goto error;
+
+		if (!g_at_result_iter_next_number(&iter, &enabled))
+			goto error;
+
+		ofono_info("AG wants indicator %d %s",
+				hf_indicator, enabled ? "enabled" : "disabled");
+
+		for (i = 0; i < info->num_hf_indicators; i++) {
+			if (info->hf_indicators[i] != hf_indicator)
+				continue;
+
+			info->hf_indicator_active_map |= enabled << i;
+		}
+
+		ofono_info("Active map: %02x", info->hf_indicator_active_map);
+	}
+
+	slc_established(sed);
+	return;
+
+error:
+	slc_failed(sed);
+}
+
+static void bind_support_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct slc_establish_data *sed = user_data;
+	struct hfp_slc_info *info = sed->info;
+	GAtResultIter iter;
+	int hf_indicator;
+
+	if (!ok)
+		goto error;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+BIND:"))
+		goto error;
+
+	if (!g_at_result_iter_open_list(&iter))
+		goto error;
+
+	while (g_at_result_iter_next_number(&iter, &hf_indicator)) {
+		if (info->num_hf_indicators >= 20)
+			goto error;
+
+		ofono_info("AG supports the following HF indicator: %d",
+				hf_indicator);
+
+		info->hf_indicators[info->num_hf_indicators] = hf_indicator;
+		info->num_hf_indicators += 1;
+	}
+
+	if (!g_at_result_iter_close_list(&iter))
+		goto error;
+
+	slc_establish_data_ref(sed);
+	g_at_chat_send(info->chat, "AT+BIND?", bind_prefix,
+				bind_query_cb, sed, slc_establish_data_unref);
+
+	return;
+
+error:
+	slc_failed(sed);
+}
+
+static void bind_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct slc_establish_data *sed = user_data;
+	struct hfp_slc_info *info = sed->info;
+
+	if (!ok) {
+		slc_failed(sed);
+		return;
+	}
+
+	slc_establish_data_ref(sed);
+	g_at_chat_send(info->chat, "AT+BIND=?", bind_prefix,
+				bind_support_cb, sed, slc_establish_data_unref);
+}
+
 static void chld_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct slc_establish_data *sed = user_data;
@@ -148,7 +257,14 @@ static void chld_cb(gboolean ok, GAtResult *result, gpointer user_data)
 
 	info->ag_mpty_features = ag_mpty_feature;
 
-	slc_established(sed);
+	if ((info->ag_features & HFP_AG_FEATURE_HF_INDICATORS) &&
+			(info->hf_features & HFP_HF_FEATURE_HF_INDICATORS)) {
+		slc_establish_data_ref(sed);
+		g_at_chat_send(info->chat, "AT+BIND=1", none_prefix,
+				bind_set_cb, sed, slc_establish_data_unref);
+	} else
+		slc_established(sed);
+
 	return;
 
 error:
@@ -319,8 +435,8 @@ static void brsf_cb(gboolean ok, GAtResult *result, gpointer user_data)
 			sprintf(str, "AT+BAC=%d", HFP_CODEC_CVSD);
 
 		slc_establish_data_ref(sed);
-		g_at_chat_send(info->chat, str, NULL, bac_cb, sed,
-						slc_establish_data_unref);
+		g_at_chat_send(info->chat, str, none_prefix, bac_cb,
+				sed, slc_establish_data_unref);
 		return;
 	}
 
