@@ -139,8 +139,17 @@ static void sim_auth_unregister(struct ofono_atom *atom)
 	struct ofono_sim_auth *sa = __ofono_atom_get_data(atom);
 
 	free_apps(sa);
+	g_free(sa->nai);
 
-	g_free(sa->pending);
+	if (sa->pending) {
+		__ofono_dbus_pending_reply(&sa->pending->msg,
+				__ofono_error_sim_not_ready(sa->pending->msg));
+
+		__ofono_sim_remove_session_watch(sa->pending->session,
+							sa->pending->watch_id);
+		g_free(sa->pending);
+		sa->pending = NULL;
+	}
 }
 
 static void sim_auth_remove(struct ofono_atom *atom)
