@@ -795,6 +795,9 @@ static void netreg_status_watch(int status, int lac, int ci, int tech,
 	if (sms->tx_source > 0)
 		return;
 
+	if (sms->flags & MESSAGE_MANAGER_FLAG_TXQ_ACTIVE)
+		return;
+
 	if (g_queue_get_length(sms->txq))
 		sms->tx_source = g_timeout_add(0, tx_next, sms);
 }
@@ -985,8 +988,7 @@ static DBusMessage *sms_send_message(DBusConnection *conn, DBusMessage *msg,
 	err = __ofono_sms_txq_submit(sms, msg_list, flags, &uuid,
 					message_queued, msg);
 
-	g_slist_foreach(msg_list, (GFunc) g_free, NULL);
-	g_slist_free(msg_list);
+	g_slist_free_full(msg_list, g_free);
 
 	if (err < 0)
 		return __ofono_error_failed(msg);
@@ -1103,23 +1105,35 @@ int __ofono_sms_txq_cancel(struct ofono_sms *sms, const struct ofono_uuid *uuid)
 	return 0;
 }
 
-static GDBusMethodTable sms_manager_methods[] = {
-	{ "GetProperties",    "",    "a{sv}",        sms_get_properties,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "SetProperty",      "sv",  "",             sms_set_property,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "SendMessage",      "ss",  "o",             sms_send_message,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "GetMessages",       "",    "a(oa{sv})",    sms_get_messages },
+static const GDBusMethodTable sms_manager_methods[] = {
+	{ GDBUS_ASYNC_METHOD("GetProperties",
+				NULL, GDBUS_ARGS({ "properties", "a{sv}" }),
+				sms_get_properties) },
+	{ GDBUS_ASYNC_METHOD("SetProperty",
+			GDBUS_ARGS({ "property", "s" }, { "value", "v" }),
+			NULL, sms_set_property) },
+	{ GDBUS_ASYNC_METHOD("SendMessage",
+			GDBUS_ARGS({ "to", "s" }, { "text", "s" }),
+			GDBUS_ARGS({ "path", "o" }),
+			sms_send_message) },
+	{ GDBUS_METHOD("GetMessages",
+			NULL, GDBUS_ARGS({ "messages", "a(oa{sv})" }),
+			sms_get_messages) },
 	{ }
 };
 
-static GDBusSignalTable sms_manager_signals[] = {
-	{ "PropertyChanged",	"sv"		},
-	{ "IncomingMessage",	"sa{sv}"	},
-	{ "ImmediateMessage",	"sa{sv}"	},
-	{ "MessageAdded",	"oa{sv}"	},
-	{ "MessageRemoved",	"o"		},
+static const GDBusSignalTable sms_manager_signals[] = {
+	{ GDBUS_SIGNAL("PropertyChanged",
+			GDBUS_ARGS({ "name", "s" }, { "value", "v" })) },
+	{ GDBUS_SIGNAL("IncomingMessage",
+			GDBUS_ARGS({ "message", "s" }, { "info", "a{sv}" })) },
+	{ GDBUS_SIGNAL("ImmediateMessage",
+			GDBUS_ARGS({ "message", "s" }, { "info", "a{sv}" })) },
+	{ GDBUS_SIGNAL("MessageAdded",
+			GDBUS_ARGS({ "path", "o" },
+						{ "properties", "a{sv}" })) },
+	{ GDBUS_SIGNAL("MessageRemoved",
+			GDBUS_ARGS({ "path", "o" })) },
 	{ }
 };
 
@@ -1157,7 +1171,8 @@ static gboolean compute_incoming_msgid(GSList *sms_list,
 static void dispatch_app_datagram(struct ofono_sms *sms,
 					const struct ofono_uuid *uuid,
 					int dst, int src,
-					unsigned char *buf, unsigned len,
+					const unsigned char *buf,
+					unsigned int len,
 					const struct sms_address *addr,
 					const struct sms_scts *scts)
 {
@@ -1169,6 +1184,7 @@ static void dispatch_app_datagram(struct ofono_sms *sms,
 	ofono_sms_datagram_notify_cb_t notify;
 	struct sms_handler *h;
 	GSList *l;
+	gboolean dispatched = FALSE;
 
 	ts = sms_scts_to_time(scts, &remote);
 	localtime_r(&ts, &local);
@@ -1180,9 +1196,15 @@ static void dispatch_app_datagram(struct ofono_sms *sms,
 		if (!port_equal(dst, h->dst) || !port_equal(src, h->src))
 			continue;
 
+		dispatched = TRUE;
+
 		notify(sender, &remote, &local, dst, src, buf, len,
 			h->item.notify_data);
 	}
+
+	if (!dispatched)
+		ofono_info("Datagram with ports [%d,%d] not delivered",
+								dst, src);
 }
 
 static void dispatch_text_message(struct ofono_sms *sms,
@@ -1403,8 +1425,7 @@ static void handle_deliver(struct ofono_sms *sms, const struct sms *incoming)
 			return;
 
 		sms_dispatch(sms, sms_list);
-		g_slist_foreach(sms_list, (GFunc) g_free, NULL);
-		g_slist_free(sms_list);
+		g_slist_free_full(sms_list, g_free);
 
 		return;
 	}
@@ -1447,7 +1468,7 @@ static inline gboolean handle_mwi(struct ofono_sms *sms, struct sms *s)
 	return discard;
 }
 
-void ofono_sms_deliver_notify(struct ofono_sms *sms, unsigned char *pdu,
+void ofono_sms_deliver_notify(struct ofono_sms *sms, const unsigned char *pdu,
 				int len, int tpdu_len)
 {
 	struct ofono_modem *modem = __ofono_atom_get_modem(sms->atom);
@@ -1594,7 +1615,7 @@ out:
 	handle_deliver(sms, &s);
 }
 
-void ofono_sms_status_notify(struct ofono_sms *sms, unsigned char *pdu,
+void ofono_sms_status_notify(struct ofono_sms *sms, const unsigned char *pdu,
 				int len, int tpdu_len)
 {
 	struct sms s;
@@ -1924,8 +1945,7 @@ static void sms_restore_tx_queue(struct ofono_sms *sms)
 		g_queue_push_tail(sms->txq, txq_entry);
 
 loop_out:
-		g_slist_foreach(backup_entry->msg_list, (GFunc)g_free, NULL);
-		g_slist_free(backup_entry->msg_list);
+		g_slist_free_full(backup_entry->msg_list, g_free);
 		g_free(backup_entry);
 	}
 
@@ -2102,16 +2122,17 @@ int __ofono_sms_txq_set_submit_notify(struct ofono_sms *sms,
 					void *data,
 					ofono_destroy_func destroy)
 {
-	struct message *m;
-	struct tx_queue_entry *entry;
+	GList *l;
+	struct tx_queue_entry *entry = g_queue_peek_tail(sms->txq);
 
-	m = g_hash_table_lookup(sms->messages, uuid);
-	if (m == NULL)
-		return -ENOENT;
+	if (memcmp(&entry->uuid, uuid, sizeof(entry->uuid))) {
+		l = g_queue_find_custom(sms->txq, uuid, entry_compare_by_uuid);
 
-	entry = message_get_data(m);
-	if (entry == NULL)
-		return -ENOTSUP;
+		if (l == NULL)
+			return -ENOENT;
+
+		entry = l->data;
+	}
 
 	tx_queue_entry_set_submit_notify(entry, cb, data, destroy);
 
